@@ -2,7 +2,7 @@ package ExtUtils::Typemaps;
 use 5.006001;
 use strict;
 use warnings;
-our $VERSION = '3.35';
+our $VERSION = '3.54';
 
 require ExtUtils::ParseXS;
 require ExtUtils::ParseXS::Constants;
@@ -378,7 +378,7 @@ sub remove_inputmap {
   return $self->_remove($xstype, $self->{input_section}, $self->{input_lookup});
 }
 
-=head2 remove_inputmap
+=head2 remove_outputmap
 
 Removes an C<OUTPUT> entry from the typemap.
 
@@ -781,7 +781,9 @@ corresponding OUTPUT code:
                 $var.context.value().size());
   ',
     'T_OUT' => '    {
-            GV *gv = newGVgen("$Package");
+            GV *gv = (GV *)sv_newmortal();
+            gv_init_pvn(gv, gv_stashpvs("$Package",1),
+                       "__ANONIO__",10,0);
             if ( do_open(gv, "+>&", 3, FALSE, 0, 0, $var) )
                 sv_setsv(
                   $arg,
@@ -924,10 +926,12 @@ sub clone {
 
 =head2 tidy_type
 
-Function to (heuristically) canonicalize a C type. Works to some
-degree with C++ types.
+Function to (heuristically) canonicalize a C type in terms of white space.
+Works to some degree with C++ types. For example,
 
-    $halfway_canonical_type = tidy_type($ctype);
+    $halfway_canonical_type = tidy_type('  int *  * const   *  ');
+
+returns C<'int ** const *'>.
 
 Moved from C<ExtUtils::ParseXS>.
 
